if not QGIS_is_live:
    from ..application import BLUE_GEO_QGIS_APPLICATION
    from ..file import Q_file_exists
    from ..logger import Q_log, Q_log_error
    from ..objects import Q_file_path_in_object
    from ..project import Q_project
    from ..QGIS import QGIS


class BLUE_GEO_QGIS_APPLICATION_DATACUBE(BLUE_GEO_QGIS_APPLICATION):
    def __init__(self):
        super().__init__("datacube", "🧊")

    def help_(self):
        Q_log(
            "datacube.label",
            "label the datacube.",
            icon=self.icon,
        )

    @property
    def label(self):
        datacube_id = Q_project.name

        filename = Q_file_path_in_object(
            filename="label.shp",
            object_name=datacube_id,
        )

        if not Q_file_exists(filename):
            Q_log_error(f"file not found: {filename}")
            Q_log(f'try running "@datacube label - {datacube_id}".')
            return

        Q_project.add_layer(
            filename=filename,
            layer_name=f"label-{datacube_id}",
            template_name="template-label",
        )


datacube = BLUE_GEO_QGIS_APPLICATION_DATACUBE()
QGIS.add_app(datacube)



import argparse
import datetime

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.catalog.default import add_default_arguments
from blue_geo.catalog.maxar_open_data.collection import MaxarOpenDataDatacube
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="query",
)
add_default_arguments(MaxarOpenDataDatacube.query_args, parser)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)

args = parser.parse_args()


success = False
if args.task == "query":
    success = MaxarOpenDataDatacube.query(
        object_name=args.object_name,
        collection_id=args.collection_id,
        lat=args.lat,
        lon=args.lon,
        radius=args.radius,
        start_date=args.start_date,
        end_date=args.end_date,
        count=args.count,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



if not QGIS_is_live:
    from .logger import Q_clear
    from .QGIS import QGIS


def Q_help(clear=False):
    if clear:
        Q_clear(log=False)

    for thing in [app for app in QGIS.app_list if app.name != "template"] + [QGIS]:
        thing.help_()



from typing import List

from blue_options.terminal import show_usage


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "catalogs"

    args = [
        "[--count 1]",
        "[--delim ,]",
        "[--log 0]",
    ]

    usage_1 = show_usage(
        [
            "@catalog list",
            f"[{options}]",
        ]
        + args,
        "list catalogs.",
        mono=mono,
    )

    # ----

    options = "collections|datacubes==datacube_classes"

    args = ["[--catalog <catalog>]"] + args

    usage_2 = show_usage(
        [
            "@catalog list",
            f"[{options}]",
        ]
        + args,
        f"list {options} in <catalog>.",
        mono=mono,
    )

    # ----

    return "\n".join(
        [
            usage_1,
            usage_2,
        ]
    )



from typing import List

if not QGIS_is_live:
    from logger import Q_log


class BLUE_GEO_QGIS_APPLICATION:
    def __init__(self, name, icon):
        self.name = name
        self.icon = icon

        self.log(self.name)

    def help_(self):
        pass

    def log(self, message):
        Q_log(message, icon=self.icon)

    # aliases

    @property
    def help(self):
        self.help_()



from blueness import module

from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def rasterize_the_label(
    datacube_id: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.rasterize_the_label({datacube_id})")

    logger.info("🪄")

    return True



if not QGIS_is_live:
    from ..logger import Q_log
    from ..application import BLUE_GEO_QGIS_APPLICATION
    from ..QGIS import QGIS


class BLUE_GEO_QGIS_APPLICATION_TEMPLATE(BLUE_GEO_QGIS_APPLICATION):
    def __init__(self):
        super().__init__("template", "🌀")

    def help_(self):
        Q_log(
            "template.func(var)",
            "func.",
            icon=self.icon,
        )

    def func(self, var: str = "🪄"):
        self.log(var)


template = BLUE_GEO_QGIS_APPLICATION_TEMPLATE()
QGIS.add_app(template)



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.catalog.generic.generic.scope import DatacubeScope


def ingest_options(mono: bool) -> str:
    return "".join(
        [
            xtra("~copy_template,dryrun,overwrite,", mono),
            "scope=<scope>",
            xtra(",upload", mono),
        ]
    )


scope_details = {
    f"scope: {DatacubeScope.help}": [
        "all: ALL files.",
        "metadata (default): any < 1 MB.",
        "raster: all raster.",
        "rgb: rgb.",
        "rgbx: rgb and what is needed to build rgb.",
        "<suffix>: any *<suffix>.",
    ]
}


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@datacube",
            "ingest",
            f"[{ingest_options(mono)}]",
            "[.|<datacube-id>]",
            "[<args>]",
        ],
        "ingest <datacube-id>/<scope>.",
        scope_details,
        mono=mono,
    )



if not QGIS_is_live:
    from logger import Q_log, Q_hr

    from tests.alias import test_aliases
    from tests.application import test_template_application
    from tests.graphics import test_graphics_screenshot
    from tests.help import test_help
    from tests.logger import test_logging
    from tests.objects import test_objects_get_name_or_path, test_objects_upload
    from tests.QGIS import test_QGIS, test_QGIS_aliases, test_QGIS_list_recent_projects
    from tests.string import test_timestamp


list_of_tests = [
    test_aliases,
    test_graphics_screenshot,
    test_help,
    test_logging,
    test_objects_get_name_or_path,
    test_objects_upload,
    test_QGIS,
    test_QGIS_aliases,
    test_QGIS_list_recent_projects,
    test_template_application,
    test_timestamp,
]


def Q_test():
    description: str = "{} test(s): {}".format(
        len(list_of_tests),
        ", ".join([test_function.__name__ for test_function in list_of_tests]),
    )

    Q_log(f"running {description} ...")
    Q_hr()

    for test_function in list_of_tests:
        Q_log(f"testing {test_function} ...")
        test_function()
        Q_hr()

    Q_log(f"ran {description}.", icon="✅")



from abcli.help.generic import help_functions as generic_help_functions

from blue_geo import ALIAS
from blue_geo.help.catalog import help_functions as help_catalog
from blue_geo.help.datacube import help_functions as help_datacube
from blue_geo.help.gdal import help_functions as help_gdal
from blue_geo.help.ingest import help_ingest
from blue_geo.help.logger import help_log
from blue_geo.help.QGIS import help_functions as help_QGIS
from blue_geo.help.watch import help_functions as help_watch

help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "catalog": help_catalog,
        "datacube": help_datacube,
        "gdal": help_gdal,
        "ingest": help_ingest,
        "log": help_log,
        "QGIS": help_QGIS,
        "watch": help_watch,
    }
)



import os
from typing import List
from glob import glob

from blue_objects.env import ABCLI_OBJECT_ROOT
from blue_objects import file, path

from blue_geo import VERSION

default_init_script: List[str] = [
    "QGIS.test",
    "QGIS.help",
]


def generate_seed(
    init_script: List[str] = default_init_script,
) -> str:
    path = file.path(__file__)

    apps_path = os.getenv(
        "BLUE_GEO_QGIS_APPS_PATH",
        os.path.join(path, "console/apps"),
    )

    list_of_files = (
        [
            os.path.join(path, f"{module}.py")
            for module in [
                "dependency",
                "console/logger",
                "console/project",
                "console/layer",
                "console/file",
                "console/file_load",
                "console/file_save",
                "console/graphics",
                "console/objects",
                "console/path",
                "console/string",
                "console/application",
                "console/seed",
                "console/help",
                "console/QGIS",
            ]
        ]
        + sorted(glob(f"{apps_path}/*.py"))
        + sorted(glob(os.path.join(path, "console/tests/*.py")))
        + [
            os.path.join(path, f"console/{module}.py")
            for module in [
                "main",
                "testing",
            ]
        ]
    )

    seed: List[str] = (
        [
            f'ABCLI_OBJECT_ROOT="{ABCLI_OBJECT_ROOT}"',
            f'BLUE_GEO_VERSION="{VERSION}"',
        ]
        + [f'exec(Path("{filename}").read_text())' for filename in list_of_files]
        + init_script
    )

    return "; ".join(seed)



from typing import List

from blue_options.terminal import show_usage

from blue_geo.help.datacube.ingest import scope_details


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--count 1]",
        "[--delim +]",
        "[--exists 1]",
        "[--log 0]",
        "[--scope <scope>]",
    ]

    return show_usage(
        [
            "@datacube list",
            "[.|<datacube-id>]",
        ]
        + args,
        "list datacube files.",
        scope_details,
        mono=mono,
    )



from blue_geo.help.catalog.browse import help_browse
from blue_geo.help.catalog.get import help_get
from blue_geo.help.catalog.list import help_list
from blue_geo.help.catalog.query import help_query


help_functions = {
    "browse": help_browse,
    "get": help_get,
    "list": help_list,
    "query": help_query,
}



import os

from blue_objects import file, README
from blue_options.help.functions import get_help

from blue_geo import NAME, VERSION, ICON, REPO_NAME
from blue_geo.catalog import get_catalog
from blue_geo.help.functions import help_functions


def build() -> bool:
    return all(
        README.build(
            items=[],
            cols=3,
            path=os.path.join(file.path(__file__), suffix),
            macros=macros,
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for suffix, macros, in [
            (catalog, {"--urls--": get_catalog(catalog).urls_as_str()})
            for catalog in [
                "copernicus",
                "EarthSearch",
                "maxar_open_data",
                "SkyFox",
                "firms",
                "ukraine_timemap",
            ]
        ]
    )



from typing import List
import json
import numpy as np

from blueness import module
from blue_options import string
from blue_objects import file, objects
from blue_objects.logger.matrix import log_matrix
from blueflow import fullname as blueflow_fullname

from blue_geo import fullname

from blue_geo import NAME

NAME = module.name(__file__, NAME)


def log_geoimage(
    object_name: str,
    filename: str,
    header: List[str] = [],
    footer: List[str] = [],
    colormap: int = -1,  # example: cv2.COLORMAP_JET
    log: bool = True,
    verbose: bool = True,
    **kwargs,
) -> bool:
    full_filename = objects.path_of(
        filename=filename,
        object_name=object_name,
    )

    success, matrix, metadata = file.load_geoimage(
        full_filename,
        log=log,
    )
    if not success:
        return success

    matrix = np.transpose(matrix, (1, 2, 0))

    metadata_as_str: List[str] = []
    try:
        for keyword, value in metadata.items():
            try:
                metadata_as_str += [f"{keyword}={json.dumps(value)}"]
            except Exception:
                metadata_as_str += [f"{keyword}=..."]
    except Exception:
        pass

    if matrix.ndim == 2:
        matrix = np.expand_dims(matrix, axis=-1)

    if matrix.shape[2] == 1:
        matrix = np.repeat(matrix, 3, axis=2)

    if matrix.shape[2] > 3:
        matrix = matrix[:, :, :3]

    range_signature: List[str] = [string.pretty_shape_of_matrix(matrix)]
    if colormap == -1 and matrix.dtype != np.uint8:
        matrix = matrix.astype(np.float64)

        for index in range(matrix.shape[2]):
            min_value = np.min(matrix[:, :, index])
            max_value = np.max(matrix[:, :, index])

            matrix[:, :, index] = (
                255
                * (matrix[:, :, index] - min_value)
                / (max_value - min_value + np.finfo(np.float64).eps)
            )

            range_signature += [f"range[{index}]: {min_value:.3f} ... {max_value:.3f}"]

        matrix = matrix.astype(np.uint8)

    return log_matrix(
        matrix=matrix,
        filename=file.add_extension(full_filename, "png"),
        header=objects.signature(
            info=filename,
            object_name=object_name,
        )
        + metadata_as_str
        + range_signature
        + header,
        footer=[
            fullname(),
            blueflow_fullname(),
        ]
        + footer,
        colormap=colormap,
        log=log,
        verbose=verbose,
        log_shape_of_matrix=False,
        **kwargs,
    )



import os

from blue_options.help.functions import get_help
from blue_objects import file, README

from blue_geo import NAME, VERSION, ICON, REPO_NAME
from blue_geo.watch.targets import jasper
from blue_geo.help.functions import help_functions
from blue_geo.watch.targets.target_list import TargetList


def build() -> bool:
    target_list = TargetList(download=True)

    return all(
        README.build(
            items=items,
            cols=cols,
            path=os.path.join(
                file.path(__file__),
                f"md/{suffix}",
            ),
            macros=macros,
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for suffix, items, cols, macros, in [
            (
                f"{target_name}.md",
                jasper.items if target_name == "Jasper" else [],
                len(jasper.list_of_dates) if target_name == "Jasper" else 3,
                {
                    "--footer--": [
                        "---",
                        "",
                        "used by: {}.".format(
                            ", ".join(
                                sorted(
                                    [
                                        "[`@geo watch`](../../)",
                                    ]
                                )
                            )
                        ),
                    ],
                    "--urls--": target_list.get(target_name).urls_as_str(),
                },
            )
            for target_name in [
                "DrugSuperLab",
                "Fagradalsfjall",
                "Hurricane-Idalia-2023",
                "Jasper",
                "Leonardo",
                "Mount-Etna",
                "Palisades",
                "burning-man-2024",
                "chilcotin-river-landslide",
            ]
        ]
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_log(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("~download,dryrun,", mono=mono),
            "filename=<filename.tif>,upload",
        ]
    )

    args = [
        "[--log <0>]",
        "[--verbose <0>]",
        "[--header <some-text>]",
        "[--footer <some-text>]",
    ]

    return show_usage(
        [
            "@geo",
            "log",
            f"[{options}]",
            "[.|<object-name>]",
        ]
        + args,
        "log <object-name>/<filename.tif>.",
        mono=mono,
    )



from blue_geo import REPO_NAME

README = {
    "objects": {
        "test_blue_geo_watch_v4-diff-chilcotin-river-landslide-test": [
            f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
        ],
        "test_blue_geo_watch_v4-modality-chilcotin-river-landslide-test": [
            f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
        ],
        "geo-watch-2024-08-31-chilcotin-c": [
            "L1C and L2A mixed",
            "`2024-07-30/2024-08-09`",
        ],
        "geo-watch-2024-09-01-chilcotin-a": [],
        "geo-watch-2024-09-01-chilcotin-c": [
            "[on reddit](https://www.reddit.com/r/bash/comments/1f9cvyx/a_bash_python_tool_to_watch_a_target_in_satellite/).",
        ],
        "geo-watch-Chilcotin-2024-11-03": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



README = {
    "objects": {
        "Sheerness-20x-2024-12-09-S8xKmn": [
            "half-blank frames, will rerun with content-ratio > 0.6.",
        ],
        "Sheerness-20x-2024-12-14-EDkXl0": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



if not QGIS_is_live:
    from .logger import Q_log
    from .objects import Q_file_path_in_object
    from .project import Q_project
    from .string import Q_timestamp

    from .mock import iface, qgis


def Q_refresh(deep=False):
    Q_log("{}refresh.".format("deep" if deep else ""))

    if deep:
        # https://api.qgis.org/api/classQgsMapCanvas.html
        iface.mapCanvas().redrawAllLayers()
    else:
        iface.mapCanvas().refresh()


def Q_screenshot(filename=""):
    filename = Q_file_path_in_object(
        filename=filename if filename else f"{Q_timestamp()}.png",
        object_name=Q_project.name,
    )

    qgis.utils.iface.mapCanvas().saveAsImage(filename)

    Q_log(filename, icon="🖼️")



Q_verbose = False

QGIS_is_live = True

if not QGIS_is_live:
    from .QGIS import QGIS
    from .seed import Q_seed
    from .mock import iface


def Q_clear(
    log: bool = True,
    dryrun: bool = False,
):
    if not dryrun:
        # https://gis.stackexchange.com/a/480025/250728
        from qgis.PyQt.QtWidgets import QDockWidget

        consoleWidget = iface.mainWindow().findChild(QDockWidget, "PythonConsole")
        consoleWidget.widget().console.clearButton.trigger()

    QGIS.intro()

    Q_seed(
        "clear",
        log=log,
        dryrun=dryrun,
    )


def Q_hr(length: int = 3):
    print(length * ". .. ... .. ")


def Q_log(message, note="", icon="🌐"):
    print(
        "{} {}{}".format(
            icon,
            (
                (f"{message:.<40}" if len(message) < 38 else f"{message}\n   {40*'.'}")
                if note
                else message
            ),
            note,
        )
    )


def Q_log_error(message):
    Q_log(message, icon="❗️")


def Q_log_warning(message):
    Q_log(message, icon="❓")



from typing import List

from blue_options.terminal import show_usage


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "catalog | raw | template"

    return show_usage(
        [
            "@datacube get",
            f"[{options}]",
            "[.|<datacube-id>]",
        ],
        "get datacube properties.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage

from blue_geo.catalog import get_catalog

from blue_geo.catalog.classes import list_of_catalogs


def help_browse(
    tokens: List[str],
    mono: bool,
) -> str:
    if tokens:
        catalog_name = tokens[0]
        catalog = get_catalog(catalog_name)

        return show_usage(
            [
                "@catalog browse",
                catalog_name,
                "|".join(catalog.url.keys()),
            ],
            f"browse {catalog_name}.",
            mono=mono,
        )

    return "\n".join([help_browse([catalog], mono) for catalog in list_of_catalogs])



import os
from tqdm import tqdm

if not QGIS_is_live:
    from .file import Q_file_exists, Q_get_file_extension
    from .logger import Q_verbose, Q_log, Q_log_error
    from .graphics import Q_refresh
    from .file_load import Q_load_yaml

    from .mock import (
        QgsRasterLayer,
        QgsProject,
        QgsMapLayerStyle,
        QgsVectorLayer,
        iface,
    )


class ABCLI_QGIS_Project(object):
    def add_layer(
        self,
        filename: str,
        layer_name: str,
        template_name: str = "",
        set_nodata: bool = False,
        nodata: int = 0,
        refresh: bool = True,
        verbose: bool = True,
    ) -> bool:
        if self.exists(layer_name, verbose=verbose):
            return True

        if Q_get_file_extension(filename) in ["geojson", "shp"]:
            layer_ = QgsVectorLayer(filename, layer_name, "ogr")
        elif filename.endswith(".tif"):
            layer_ = QgsRasterLayer(filename, layer_name)
        else:
            Q_log_error(f"cannot load {filename}.")
            return False

        if not layer_.isValid():
            Q_log_error(f"invalid layer: {filename}.")
            return False

        QgsProject.instance().addMapLayer(layer_)

        if layer_ and layer_.dataProvider() and set_nodata:
            layer_.dataProvider().setNoDataValue(1, nodata)
            layer_.dataProvider().reloadData()
            layer_.triggerRepaint()

        if template_name:
            template_layer = self.get_layer(template_name)
            if not len(template_layer):
                Q_log_error(f"template not found: {template_name}.")
                return False

            # https://gis.stackexchange.com/a/357206/210095
            source_style = QgsMapLayerStyle()
            source_style.readFromLayer(template_layer[0])
            source_style.writeToLayer(layer_)
            layer_.triggerRepaint()

        Q_log(
            f"{layer_name} += {template_name}",
            icon="🎨",
        )

        if refresh:
            Q_refresh()

        return True

    def exists(
        self,
        layer_name: str,
        verbose: bool = True,
    ) -> bool:
        list_of_layers = self.get_layer(layer_name=layer_name)

        if len(list_of_layers) > 0:
            if verbose:
                Q_log(layer_name, icon="✅")
            return True

        return False

    @property
    def filename(self):
        return QgsProject.instance().fileName()

    def get_layer(self, layer_name):
        return QgsProject.instance().mapLayersByName(layer_name)

    @property
    def list_of_layers(
        self,
        aux: bool = False,
    ):
        output = [layer.name() for layer in QgsProject.instance().mapLayers().values()]

        if not aux:
            output = [
                layer_name
                for layer_name in output
                if not layer_name.startswith("Google")
                and not layer_name.startswith("template")
            ]

        if Q_verbose:
            Q_log(
                f"{len(output)} layer(s)",
                ", ".join(output),
                icon="🔎",
            )

        return output

    @property
    def metadata(self):
        filename = os.path.join(self.path, "metadata.yaml")

        if not Q_file_exists(filename):
            return {"error": f"{filename}: file not found."}

        return Q_load_yaml(filename)

    @property
    def name(self):
        return QgsProject.instance().homePath().split(os.sep)[-1]

    @property
    def path(self):
        return QgsProject.instance().homePath()

    def reload(self):
        # https://gis.stackexchange.com/a/449101/210095
        for layer_ in tqdm(QgsProject.instance().mapLayers().values()):
            layer_.dataProvider().reloadData()

    def remove_layer(
        self,
        layer_name: str,
        refresh: bool = True,
    ):
        Q_log(layer_name, icon="🗑️")

        for layer_ in QgsProject.instance().mapLayersByName(layer_name):
            QgsProject.instance().removeMapLayer(layer_.id())
            Q_log(layer_name, icon="➖")

        if refresh:
            Q_refresh()

    def trim_empty_groups(self):
        root = QgsProject.instance().layerTreeRoot()
        groups_to_check = [root]

        while groups_to_check:
            group = groups_to_check.pop(0)
            children = group.children()
            empty = all(
                child.nodeType() == 0 and child.children() == [] for child in children
            )

            if empty and group != root:
                parent = group.parent()
                parent.removeChildNode(group)
            else:
                groups_to_check.extend(
                    [child for child in children if child.nodeType() == 0]
                )


Q_project = ABCLI_QGIS_Project()



from typing import List

from blue_options.terminal import show_usage


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    what = "is_STAC|url:<...>|url_args|list_of_args"
    args = ["[--catalog <catalog>]"]
    return show_usage(
        [
            "@catalog get",
            f"[{what}]",
        ]
        + args,
        "get catalog properties.",
        mono=mono,
    )



README = {
    "objects": {
        "geo-watch-DrugSuperLab-2024-11-19-13954": [
            "known issues: successive frames may have different projections."
        ],
        "DrugSuperLab-2024-12-08-pGErp2": [],
        "DrugSuperLab-2024-12-09-ZnmC5L": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



from typing import List

from blue_options.terminal import show_usage


def help_crop(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "download,dryrun,suffix=<suffix>"
    return show_usage(
        [
            "@datacube crop",
            f"[{options}]",
            "[..|<object-name>]",
            "[.|<datacube-id>]",
        ],
        "crop <datacube-id> by <object-name>/target/shape.geojson -> <datacube-id>-DERIVED-crop-<suffix>.",
        mono=mono,
    )



from blue_geo.help.datacube.crop import help_crop
from blue_geo.help.datacube.generate import help_generate
from blue_geo.help.datacube.get import help_get
from blue_geo.help.datacube.ingest import help_ingest, ingest_options, scope_details
from blue_geo.help.datacube.label import help_label
from blue_geo.help.datacube.list import help_list

help_functions = {
    "crop": help_crop,
    "generate": help_generate,
    "get": help_get,
    "ingest": help_ingest,
    "label": help_label,
    "list": help_list,
}



from typing import Dict, List
import geopandas as gpd
from functools import reduce
from tqdm import tqdm

from blue_objects import file, objects

from blue_geo.env import BLUE_GEO_WATCH_TARGET_LIST
from blue_geo.watch.targets.target import Target
from blue_geo.logger import logger


class TargetList:
    object_name = BLUE_GEO_WATCH_TARGET_LIST

    def __init__(
        self,
        load: bool = True,
        download: bool = False,
        catalog: str = "",
    ) -> None:
        self.list_of_targets: Dict[str, Target] = {}

        if load:
            assert self.load(download)

        if catalog:
            self.list_of_targets = {
                target_name: target
                for target_name, target in self.list_of_targets.items()
                if target.catalog == catalog
            }

    def download(self) -> bool:
        return objects.download(
            object_name=self.object_name,
            filename="metadata.yaml",
        )

    @classmethod
    def filename(cls) -> str:
        return objects.path_of(
            filename="metadata.yaml",
            object_name=cls.object_name,
        )

    def get(
        self,
        target_name: str,
        including_versions: bool = True,
    ) -> Target:
        if not target_name:
            return Target()

        if target_name in self.list_of_targets:
            return self.list_of_targets[target_name]

        if not including_versions:
            return Target()

        for existing_target in self.list_of_targets.values():
            if target_name.startswith(existing_target.name):
                return existing_target.get_version(
                    target_name,
                    is_full_name=True,
                )

        return Target()

    def get_list(
        self,
        catalog_name: str = "",
        collection: str = "",
        including_versions: bool = True,
    ) -> List[str]:
        return sorted(
            reduce(
                lambda x, y: x + y,
                [
                    [target.name]
                    + (
                        [f"{target.name}-{version}" for version in target.versions]
                        if including_versions
                        else []
                    )
                    for target in self.list_of_targets.values()
                    if (target.catalog == catalog_name or not catalog_name)
                    and (target.collection == collection or not collection)
                ],
                [],
            )
        )

    def load(
        self,
        download: bool = False,
    ) -> bool:
        if download and not self.download():
            return False

        _, targets = file.load_yaml(self.filename(), ignore_error=True)

        self.list_of_targets = {
            target_name: Target.from_dict(
                target_name,
                target_data,
            )
            for target_name, target_data in targets.items()
        }

        return True

    def save(self, object_name: str) -> bool:
        list_of_targets = self.get_list(including_versions=True)

        logger.info(f"saving {len(list_of_targets)} target(s): {object_name}")

        gdf = gpd.GeoDataFrame(
            {
                "id": [str(index + 1) for index in range(len(list_of_targets))],
                "name": [
                    self.get(target_name=target_name).name
                    for target_name in list_of_targets
                ],
                "description": [
                    self.get(target_name=target_name).description
                    for target_name in list_of_targets
                ],
                "catalog": [
                    self.get(target_name=target_name).catalog
                    for target_name in list_of_targets
                ],
                "collection": [
                    self.get(target_name=target_name).collection
                    for target_name in list_of_targets
                ],
                "geometry": [
                    self.get(target_name=target_name).polygon
                    for target_name in list_of_targets
                ],
            },
            crs="EPSG:4326",
        )

        return file.save_geojson(
            objects.path_of("target/shape.geojson", object_name),
            gdf,
            log=True,
        )

    def test(self) -> bool:
        list_of_targets = self.get_list(
            including_versions=True,
        )
        logger.info(f"testing {len(list_of_targets)} target(s).")

        for target_name in tqdm(list_of_targets):
            target = self.get(
                target_name,
                including_versions=True,
            )
            if not target.name:
                logger.error(f"bad target: {target_name}")
                return False

            try:
                logger.info(target.one_liner)
            except Exception as e:
                logger.error(f"bad target: {target_name} - {e}")
                return False

        return True



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.datacube.label.rasterize import rasterize_the_label
from blue_geo.datacube.label.sync import sync_the_label
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="rasterize | sync",
)
parser.add_argument(
    "--datacube_id",
    type=str,
    default="",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "rasterize":
    success = rasterize_the_label(
        datacube_id=args.datacube_id,
        verbose=args.verbose == 1,
    )
elif args.task == "sync":
    success = sync_the_label(
        datacube_id=args.datacube_id,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os

if not QGIS_is_live:
    from .graphics import Q_refresh
    from .logger import Q_log_error, Q_log
    from .project import Q_project

    from .mock import iface

    ABCLI_OBJECT_ROOT = ""


class ABCLI_QGIS_Layer(object):
    @property
    def filename(self):
        try:
            return iface.activeLayer().dataProvider().dataSourceUri()
        except:
            Q_log_error("layer.filename not found.")
            return ""

    @property
    def name(self):
        filename = self.filename
        return filename.split(os.sep)[-1].split(".")[0] if filename else ""

    @property
    def object_name(self):
        filename = self.filename
        if not filename:
            return ""

        if ABCLI_OBJECT_ROOT not in filename:
            return ""

        tokens = filename.split(f"{ABCLI_OBJECT_ROOT}/", 1)[1].split("/")
        if not tokens:
            return ""

        return tokens[0]

    @property
    def path(self):
        return os.path.dirname(self.filename)


Q_layer = ABCLI_QGIS_Layer()



from typing import List

from blue_options.terminal import show_usage, xtra


def help_label(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("~download,dryrun,~QGIS,~rasterize,~sync,", mono=mono),
            "upload",
        ]
    )

    return show_usage(
        [
            "@datacube",
            "label",
            f"[{options}]",
            "[.|<datacube-id>]",
        ],
        "label <datacube-id>.",
        mono=mono,
    )



README = {
    "objects": {
        "bellingcat-2024-09-27-nagorno-karabakh-2024-10-01-c-b": [],
        "bellingcat-2024-09-27-nagorno-karabakh-b": [],
        "bellingcat-2024-09-27-nagorno-karabakh-6X-a": [],
        "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-05-b": [],
        "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-06-a": [],
        "bellingcat-2024-09-27-nagorno-karabakh-6X-2024-12-14-EUUpS1": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



from typing import List, Dict
from blue_objects.env import ABCLI_PUBLIC_PREFIX

list_of_dates: List[str] = [
    "2024_7_20",
    "2024_7_22",
    "2024_7_27",
]

date_items: Dict[str, List[str]] = {
    date: [
        "`{}`".format(date.replace("_", "-")),
        f"![image]({ABCLI_PUBLIC_PREFIX}/geo-watch-2024-09-06-Jasper-a/11_U_MU_{date}_0_TCI.png)",
    ]
    for date in list_of_dates
}

items: List[str] = [
    date_items[date][index]
    for index in range(len(date_items[list_of_dates[0]]))
    for date in list_of_dates
]

README = {
    "objects": {
        "geo-watch-2024-09-06-Jasper-a": [],
        "geo-watch-Jasper-2024-11-03": [],
    },
}



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.help.datacube.ingest import ingest_options
from blue_geo.catalog.functions import get_datacube_class_in_catalog
from blue_geo.catalog.default import as_list_of_args
from blue_geo.catalog.classes import list_of_catalogs
from blue_geo.catalog.functions import get_list_of_datacube_classes
from blue_geo.help.datacube import ingest_options as datacube_ingest_options
from blue_geo.help.datacube import scope_details


def help_query(
    tokens: List[str],
    mono: bool,
) -> str:
    if not tokens:
        return "\n".join(
            [
                help_query([token], mono)
                for token in ["ingest", "read"] + list_of_catalogs
            ]
        )

    if tokens[0] == "ingest":
        return help_query_ingest(tokens[1:], mono=mono)

    if tokens[0] == "read":
        return help_query_read(tokens[1:], mono=mono)

    catalog_name = tokens[0]

    if len(tokens) < 2:
        return "\n".join(
            [
                help_query([catalog_name, collection_name], mono)
                for collection_name in [
                    datacube_class.name
                    for datacube_class in get_list_of_datacube_classes(
                        catalog_class=catalog_name
                    )
                ]
            ]
        )

    datacube_class_name = tokens[1]
    datacube_class = get_datacube_class_in_catalog(
        catalog_name,
        datacube_class_name,
    )
    args = as_list_of_args(datacube_class.query_args)
    options = f"dryrun,{datacube_class_name},select,upload"

    return show_usage(
        [
            f"@catalog query {catalog_name}",
            f"[{options}]",
            f"[ingest,{ingest_options(mono)}]",
            "[-|<object-name>]",
        ]
        + args,
        f"{catalog_name}/{datacube_class_name} -query-> <object-name>.",
        {
            "scope: @datacube ingest help.": "",
        },
        mono=mono,
    )


def help_query_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("download,index=<index>", mono=mono)

    return show_usage(
        [
            "@catalog",
            "query",
            "ingest",
            f"[{options}]",
            "[.|<query-object-name>]",
            f"[{datacube_ingest_options(mono)}]",
        ],
        "ingest the datacubes in the query.",
        scope_details,
        mono=mono,
    )


def help_query_read(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "all,download,len"

    args = [
        "[--count <count>]",
        "[--delim <delim>]",
        "[--offset <offset>]",
        "[--prefix <prefix>]",
        "[--suffix <suffix>]",
        "[--contains <contains>]",
        "[--notcontains <not-contains>]",
    ]

    return show_usage(
        [
            "@catalog query read",
            f"[{options}]",
            "[.|<query-object-name>]",
        ]
        + args,
        "read the query.",
        mono=mono,
    )



def QgsRasterLayer(filename: str, layer_name: str):
    return None


class QgsProjectClass:
    def instance():
        return None


QgsProject = QgsProjectClass()


def QgsMapLayerStyle():
    return None


def QgsVectorLayer(filename: str, layer_name: str, driver: str):
    return None


class ifaceClass:
    def activeLayer():
        return None


iface = ifaceClass()


class QgsSettings:
    pass


class qgis_class:
    pass


qgis = qgis_class()



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.datacube.modalities import options as modality_options


def help_generate(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono)

    args = [
        "[--modality <{}>]".format("|".join(modality_options)),
        "[--overwrite <1>]",
    ]

    return show_usage(
        [
            "@datacube generate",
            f"[{options}]",
            "[.|<datacube-id>]",
        ]
        + args,
        "generate datacube/<modality>.",
        mono=mono,
    )



import argparse
from types import ModuleType

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.objects import special_objects
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="get | ingest",
)
parser.add_argument(
    "--object_name",
    default="",
    type=str,
    help=", ".join(special_objects.keys()),
)
parser.add_argument(
    "--version",
    type=str,
    default="",
    help="defaults to <object_name>.version",
)
parser.add_argument(
    "--what",
    default="template_name",
    type=str,
    help="template_name | version",
)
parser.add_argument(
    "--default",
    default="void",
    type=str,
)
parser.add_argument(
    "--overwrite",
    default=0,
    type=int,
    help="0|1",
)

args = parser.parse_args()

success = False
if args.task == "get":
    success = True
    output = args.default

    if args.object_name in special_objects:
        object_module: ModuleType = special_objects[args.object_name]

        if args.what == "template_name":
            output = object_module.template_name
        elif args.what == "version":
            output = object_module.version

    print(output)
elif args.task == "ingest":
    if args.object_name in special_objects:
        object_module: ModuleType = special_objects[args.object_name]

        version = args.version
        if not version:
            version = object_module.version

        success = object_module.ingest(
            object_name=args.object_name,
            version=version,
            overwrite=args.overwrite == 1,
        )
    else:
        logger.error(
            "{}: object not found, expected one of {}.".format(
                args.object_name,
                ", ".join(special_objects.keys()),
            )
        )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List

from blue_options.terminal import show_usage


def help_get_crs(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@gdal",
            "get_crs",
            "<filename>",
        ],
        "get <filename> crs.",
        mono=mono,
    )


def help_install(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@gdal",
            "install",
        ],
        "install gdal.",
        mono=mono,
    )


def help_version(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@gdal",
            "version",
        ],
        "show gdal version.",
        mono=mono,
    )


help_functions = {
    "get_crs": help_get_crs,
    "install": help_install,
    "version": help_version,
}



import os
import shutil

if not QGIS_is_live:
    from logger import Q_log_error, Q_log


def Q_add_extension(filename: str, extension: str) -> str:
    filename, _ = os.path.splitext(filename)

    return f"{filename}.{extension}"


def Q_file_exists(filename: str) -> bool:
    return os.path.isfile(filename)


def Q_get_file_extension(filename: str) -> str:
    return os.path.splitext(filename)[1][1:]


def Q_get_file_name(filename: str) -> str:
    _, filename = os.path.split(filename)

    return filename if "." not in filename else ".".join(filename.split(".")[:-1])


def Q_get_file_name_and_extension(filename: str) -> str:
    return os.path.basename(filename)


def Q_get_file_path(filename: str) -> str:
    return os.path.split(filename)[0]


def Q_copy_file(
    source_filename: str,
    destination_filename: str,
) -> bool:
    try:
        # https://stackoverflow.com/a/8858026
        # better choice: copy2
        shutil.copyfile(source_filename, destination_filename)
    except Exception as e:
        Q_log_error(e)
        return False

    Q_log(f"{source_filename} -> {destination_filename}")
    return True



import os

if not QGIS_is_live:
    from .logger import Q_log
    from .layer import Q_layer
    from .path import Q_open_path
    from .project import Q_project
    from .seed import Q_seed

    ABCLI_OBJECT_ROOT = ""


def Q_get_thing_name_or_path(thing, property: str) -> str:
    assert property in ["name", "path"], f"invalid property: {property}"

    return_project: bool = True
    if thing in ["project", "qgz", Q_project, "object", object]:
        pass
    elif thing in ["layer", Q_layer]:
        return_project = False
    elif isinstance(thing, str) and thing:
        return thing

    if return_project:
        return Q_project.name if property == "name" else Q_project.path

    return Q_layer.object_name if property == "name" else Q_layer.path


def Q_get_thing_name(thing) -> str:
    return Q_get_thing_name_or_path(
        thing,
        property="name",
    )


def Q_get_thing_path(thing) -> str:
    return Q_get_thing_name_or_path(
        thing,
        property="path",
    )


def Q_object_path(
    object_name: str,
    create: bool = True,
) -> str:
    output = os.path.join(ABCLI_OBJECT_ROOT, object_name)

    if create:
        os.makedirs(output, exist_ok=True)

    return output


def Q_file_path_in_object(
    filename,
    object_name="",
    create: bool = True,
) -> str:
    return os.path.join(
        Q_object_path(
            object_name=object_name,
            create=create,
        ),
        filename,
    )


def Q_open(
    thing="object",
    dryrun: bool = False,
):
    Q_open_path(
        Q_get_thing_path(thing=thing),
        dryrun=dryrun,
    )


def Q_upload(
    thing="object",
    dryrun: bool = False,
):
    Q_seed(
        command=[
            "abcli_upload",
            f"filename={Q_project.name}.qgz" if thing == "qgz" else "-",
            Q_get_thing_name(thing),
        ],
        dryrun=dryrun,
    )



from blueness import module
import glob
from tqdm import tqdm

from blue_objects import objects, file

from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def sync_the_label(
    datacube_id: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.sync_the_label({datacube_id})")

    label_filename = objects.path_of(
        object_name=datacube_id,
        filename="label.shp",
    )
    if file.exists(label_filename):
        logger.info(f"✅ {label_filename}")
        return True

    template_filename = objects.path_of(
        object_name=datacube_id,
        filename="template/label.shp",
    )
    if not file.exists(template_filename):
        logger.error(f"template not found: {template_filename}.")
        return False

    from blue_objects.storage import instance as storage

    if storage.exists(object_name=f"{datacube_id}/label.shp"):
        logger.info(f"☁️ {label_filename}")

        for filename in tqdm(glob.glob(file.add_extension(template_filename, "*"))):
            extension = file.extension(filename)

            if not storage.download_file(
                object_name=f"bolt/{datacube_id}/label.{extension}",
                filename="object",
            ):
                return False

        return True

    logger.info("copying the template...")
    for filename in tqdm(glob.glob(file.add_extension(template_filename, "*"))):
        if not file.copy(
            filename,
            file.add_extension(
                label_filename,
                file.extension(filename),
            ),
        ):
            return False

    return True



from typing import List
import os
import re
import yaml


if not QGIS_is_live:
    from dependency import list_of_dependencies
    from .application import BLUE_GEO_QGIS_APPLICATION
    from .help import Q_help
    from .file_save import Q_save_yaml
    from .graphics import Q_screenshot
    from .logger import Q_log, Q_verbose, Q_clear
    from .mock import QgsSettings
    from .objects import Q_get_thing_path, Q_upload, Q_open
    from .path import Q_open_path
    from .project import Q_project
    from .testing import Q_test

    ABCLI_OBJECT_ROOT = ""

    BLUE_GEO_VERSION = "1.1.1"

Q_NAME = "Blue-GEO QGIS"
Q_ALIAS = "Q"


# https://qgis.org/pyqgis/master/core/QgsSettings.html#qgis.core.QgsSettings.allKeys
# https://docs.qgis.org/3.28/en/docs/pyqgis_developer_cookbook/settings.html
def Q_list_recent_projects() -> str:
    settings = QgsSettings()

    list_of_filenames = [
        settings.value(key)
        for key in settings.allKeys()
        if re.match(r"UI/recentProjects/(\d+)/path", key)
    ]

    list_of_projects = [
        filename.split(f"{ABCLI_OBJECT_ROOT}/", 1)[1].split("/")[0]
        for filename in list_of_filenames
        if ABCLI_OBJECT_ROOT in filename
    ]

    for filename in list_of_filenames:
        list_of_projects += list_of_dependencies(
            filename=filename,
            ABCLI_OBJECT_ROOT=ABCLI_OBJECT_ROOT,
            verbose=Q_verbose,
        )

    list_of_projects = list(set(list_of_projects))

    Q_save_yaml(
        os.path.join(ABCLI_OBJECT_ROOT, "QGIS-recent.yaml"),
        list_of_projects,
    )

    Q_log(f"{len(list_of_projects)} recent project(s).")

    return ",".join(list_of_projects)


class ABCLI_QGIS:
    def __init__(self):
        self.app_list: List[BLUE_GEO_QGIS_APPLICATION] = []

    def add_app(self, app: BLUE_GEO_QGIS_APPLICATION):
        self.app_list += [app]

    def help_(self):
        Q_log("Q.clear", "clear Python Console.")
        Q_log("Q.list_recent_projects", "list recent projects.")
        Q_log("Q.open", "open project in Finder.")
        Q_log("Q.screenshot", "screenshot.")
        Q_log("Q.test", "test Q.")
        Q_log("Q.upload", "upload project.")

    def intro(self):
        Q_log(self.version)

        for app in [app for app in self.app_list if app.name != "template"]:
            Q_log(f"{app.icon} {app.name}")

        Q_log('Type in "Q.help" for help.')

    @property
    def version(self) -> str:
        return "{}-{} ({})".format(
            Q_NAME,
            BLUE_GEO_VERSION,
            Q_ALIAS,
        )

    # aliases

    @property
    def clear(self):
        Q_clear(log=False)

    @property
    def help(self):
        Q_help(clear=True)

    @property
    def list_recent_projects(self):
        Q_list_recent_projects()

    @property
    def open(self):
        Q_open(thing=Q_project)

    @property
    def screenshot(self):
        Q_screenshot()

    @property
    def test(self):
        Q_test()

    @property
    def upload(self):
        Q_upload(thing=Q_project)


QGIS = ABCLI_QGIS()

Q = QGIS



import argparse

from blueness import module
from blue_objects import objects
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.logger.geoimage import log_geoimage
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="log_geoimage",
)
parser.add_argument(
    "--filename",
    type=str,
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--log",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--header",
    type=str,
    default="",
)
parser.add_argument(
    "--footer",
    type=str,
    default="",
)

args = parser.parse_args()

success = False
if args.task == "log_geoimage":
    success = log_geoimage(
        filename=args.filename,
        object_name=args.object_name,
        header=[args.header] if args.header else [],
        footer=[args.footer] if args.footer else [],
        log=args.log == 1,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Tuple, Dict, Any, List
import datetime

from blue_objects.metadata import post_to_object

from blue_geo.catalog.maxar_open_data.classes import MaxarOpenDataCatalog
from blue_geo.catalog.generic import GenericDatacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo import env
from blue_geo.logger import logger


class MaxarOpenDataDatacube(GenericDatacube):
    name = "collection"

    catalog = MaxarOpenDataCatalog()

    query_args = {
        "collection_id": {
            "default": "WildFires-LosAngeles-Jan-2025",
            "help": "WildFires-LosAngeles-Jan-2025 | ...",
        },
        "lat": {
            "type": float,
            "default": -1,
            "help": "<51.83>",
        },
        "lon": {
            "type": float,
            "default": -1,
            "help": "<-122.78>",
        },
        "radius": {
            "type": float,
            "default": env.MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS,
            "help": f"<{env.MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS}>",
        },
        "start_date": {
            "default": (datetime.datetime.now() - datetime.timedelta(days=14)).strftime(
                "%Y-%m-%d"
            ),
            "help": "<yyyy-mm-dd>",
        },
        "end_date": {
            "default": (datetime.datetime.now()).strftime("%Y-%m-%d"),
            "help": "<yyyy-mm-dd>",
        },
        "count": {
            "default": -1,
            "type": int,
            "help": "-1 | <value>",
        },
    }

    QGIS_template = env.BLUE_GEO_QGIS_TEMPLATE_MAXAR_OPEN_DATA

    def ingest(
        self,
        dryrun: bool = False,
        overwrite: bool = False,
        scope: str = "metadata",
    ) -> Tuple[bool, Any]:
        success, output = super().ingest(dryrun, overwrite, scope)
        if not success:
            return success, output

        return (
            self.catalog.client.ingest(
                datacube_id=self.datacube_id,
                list_of_assets=self.list_of_files(
                    scope=DatacubeScope(scope),
                    return_asset_names=True,
                ),
            ),
            {},
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        return_asset_names: bool = False,
        verbose: bool = False,
    ) -> List[str]:
        success, item = self.catalog.client.get_item(
            datacube_id=self.datacube_id,
            log=verbose,
        )
        if not success:
            return []

        list_of_assets = [
            {
                "filename": self.catalog.client.get_filename(
                    item=item,
                    filename=asset.href,
                ),
                "asset_name": asset_name,
            }
            for asset_name, asset in item.assets.items()
        ]

        output = scope.filter(
            list_of_assets,
            needed_for_rgb=lambda filename: filename.endswith("-visual.tif"),
            is_rgb=lambda filename: filename.endswith("-visual.tif"),
            verbose=verbose,
        )

        if return_asset_names:
            output = [
                asset["asset_name"]
                for asset in list_of_assets
                if asset["filename"] in output
            ]

        return output

    @classmethod
    def parse_datacube_id(cls, datacube_id: str) -> Tuple[
        bool,
        Dict[str, Any],
    ]:
        success, _ = super().parse_datacube_id(datacube_id)
        if not success:
            return False, {}

        success, item_id, collection_id = cls.catalog.client.parse_datacube_id(
            datacube_id=datacube_id,
            log=False,
        )
        return success, {
            "item_id": item_id,
            "collection_id": collection_id,
        }

    @classmethod
    def query(
        cls,
        object_name: str,
        collection_id: str,
        start_date: str,
        end_date: str,
        lat: float = -1,
        lon: float = -1,
        radius: float = env.MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS,
        count: int = -1,
    ) -> bool:
        logger.info(f"🔎 {cls.__name__}.query -> {object_name}")

        list_of_items = cls.catalog.client.query(
            collection_id=collection_id,
            start_date=datetime.datetime.strptime(start_date, "%Y-%m-%d"),
            end_date=datetime.datetime.strptime(end_date, "%Y-%m-%d"),
            lat=lat,
            lon=lon,
            radius=radius,
            count=count,
            log=True,
        )

        list_of_datacube_ids: List[str] = [
            cls.catalog.client.get_datacube_id(
                item=item,
                collection_id=collection_id,
            )
            for item in list_of_items
        ]

        logger.info(f"{len(list_of_datacube_ids)} datacubes(s) found.")
        for index, datacube_id in enumerate(list_of_datacube_ids):
            logger.info(f"🧊 {index+1:02}: {datacube_id}")

        return post_to_object(object_name, "datacube_id", list_of_datacube_ids)



from blue_geo import REPO_NAME

README = {
    "objects": {
        "test_blue_geo_watch_v4-diff-Leonardo-test": [
            f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
        ],
        "test_blue_geo_watch_v4-modality-Leonardo-test": [
            f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)"
        ],
        "geo-watch-2024-09-30-Leonardo-g": [],
        "geo-watch-Leonardo-2024-10-05-a": [],
        "geo-watch-Leonardo-2024-10-06-a": [],
        "geo-watch-2024-10-27-16-17-36-12059": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



from typing import List, Dict

from blue_geo.catalog.generic import GenericCatalog
from blue_geo.catalog.maxar_open_data.client import MaxarOpenDataClient


class MaxarOpenDataCatalog(GenericCatalog):
    name: str = "maxar_open_data"

    url: Dict[str, str] = {
        "home": "https://www.maxar.com/open-data",
    }

    def __init__(self):
        self.client = MaxarOpenDataClient()

        self.list_of_collections: List[str] = []

    def get_list_of_collections(
        self,
        use_cache: bool = True,
    ) -> List[str]:
        if not use_cache:
            self.list_of_collections = []

        if not self.list_of_collections:
            self.list_of_collections = self.client.get_list_of_collections()

        return self.list_of_collections



import yaml
from typing import List, Any

if not QGIS_is_live:
    from .logger import Q_log


def Q_save_text(
    filename: str,
    text: List[str],
    log: bool = True,
):
    with open(filename, "w") as fp:
        fp.writelines([string + "\n" for string in text])

    if log:
        Q_log(
            "Q_save_text: {:,} line(s) -> {}".format(
                len(text),
                filename,
            )
        )


def Q_save_yaml(
    filename: str,
    data: Any,
    log=True,
):
    with open(filename, "w") as file:
        yaml.dump(data, file)

    Q_log(f"-> {filename}")



from typing import Any, List, Tuple
import pystac
import datetime
import pathlib
from tqdm import tqdm
from shapely.geometry import Point, Polygon

from blue_objects import objects, file, metadata

from blue_geo import env
from blue_geo.logger.geoimage import log_geoimage
from blue_geo.catalog.generic.generic.scope import raster_suffix
from blue_geo.logger import logger

URL = "https://maxar-opendata.s3.amazonaws.com/events/catalog.json"


class MaxarOpenDataClient:
    def __init__(
        self,
        verbose: bool = False,
    ):
        self.verbose = verbose

        self.catalog = pystac.Catalog.from_file(URL)

        if verbose:
            logger.info(self.catalog)

    def get_list_of_collections(
        self,
        log: bool = False,
    ) -> List[str]:
        list_of_collections = [
            collection.id for collection in self.catalog.get_children()
        ]

        if self.verbose or log:
            logger.info(
                "{} collection(s): {}".format(
                    len(list_of_collections),
                    ", ".join(list_of_collections),
                )
            )

        return list_of_collections

    def get_collection(
        self,
        collection_id: str,
        log: bool = False,
    ) -> Any:
        collection = self.catalog.get_child(collection_id)

        if self.verbose or log:
            logger.info(collection)

        if (collection is None) or (not self.verbose and not log):
            return collection

        logger.info(list(collection.get_children()))

        for child in collection.get_children():
            logger.info(f"{child.id}")
            for item in child.get_items():
                logger.info(
                    "    {}: {}".format(
                        item.id,
                        item.properties["datetime"],
                    )
                )

        return collection

    def get_datacube_id(
        self,
        item: Any,
        collection_id: str,
        log: bool = False,
        verbose: bool = False,
    ) -> str:
        datacube_id = (
            f"datacube-maxar_open_data-{collection_id}-{item.id.replace('/','-')}"
        )

        if log or self.verbose:
            logger.info(f"item: {item.id}")
            logger.info(item.to_dict())
            for asset in item.assets:
                logger.info("{}: {}".format(asset, item.assets[asset].href))

            logger.info(f"🧊 datacube-id: {datacube_id}")

        return datacube_id

    def parse_datacube_id(
        self,
        datacube_id: str,
        log: bool = True,
        verbose: bool = False,
    ) -> Tuple[bool, str, str]:
        collection_id: str = ""
        item_id: str = ""

        if not datacube_id.startswith("datacube-maxar_open_data-"):
            if log:
                logger.error(f"Invalid datacube-id: {datacube_id}")
            return False, collection_id, item_id

        list_of_collections = self.get_list_of_collections(log=verbose)

        candidate_collections = [
            collection_id
            for collection_id in list_of_collections
            if datacube_id.startswith(f"datacube-maxar_open_data-{collection_id}-")
        ]
        if not candidate_collections:
            if log:
                logger.error(f"collection not found: {datacube_id}.")
            return False, collection_id, item_id
        if len(candidate_collections) > 1:
            if log:
                logger.warning(
                    "{} possible collection(s): {}".format(
                        len(candidate_collections), ", ".join(candidate_collections)
                    )
                )

        collection_id = candidate_collections[0]
        if log:
            logger.info(f"collection_id: {collection_id}")

        suffix = datacube_id.split(f"datacube-maxar_open_data-{collection_id}-", 1)[1]
        if "-DERIVED-" in suffix:
            suffix = suffix.split("-DERIVED-", 1)[0]

        item_id = suffix.replace("-", "/")
        if log:
            logger.info(f"item_id: {item_id}")

        return True, collection_id, item_id

    def get_filename(self, item, filename: str) -> str:
        return "{}-{}".format(
            item.id.replace("/", "-"),
            (filename.split("./", 1)[1] if filename.startswith("./") else filename),
        )

    def get_item(
        self,
        datacube_id: str,
        log: bool = False,
        read_from_cache: bool = env.MAXAR_OPEN_DATA_CLIENT_CACHE_ITEMS,
        write_to_cache: bool = env.MAXAR_OPEN_DATA_CLIENT_CACHE_ITEMS,
    ) -> Tuple[bool, Any]:
        cache_filename = objects.path_of(
            object_name=datacube_id,
            filename="item.bin",
        )

        if read_from_cache and file.exists(cache_filename):
            success, item = file.load(cache_filename, ignore_error=True)
            if success:
                return success, item

        success, collection_id, item_id = self.parse_datacube_id(
            datacube_id=datacube_id,
            log=log,
            verbose=log,
        )
        if not success:
            return success, None

        collection = self.get_collection(
            collection_id=collection_id,
            log=log,
        )
        if collection is None:
            return False, None

        list_of_items = []
        for child in collection.get_children():
            list_of_items += [item for item in child.get_items() if item.id == item_id]
        if not list_of_items:
            logger.error(f"item not found: {item_id}.")
            return False, None
        if len(list_of_items) > 1:
            logger.warning(
                "{} possible item(s): {}".format(
                    len(list_of_items), ", ".join(list_of_items)
                )
            )
        item = list_of_items[0]
        if log:
            logger.info(f"item: {item}.")

        if write_to_cache:
            file.save(cache_filename, item)

        return True, item

    def ingest(
        self,
        datacube_id: str,
        list_of_assets: List[str] = ["visual"],
        log: bool = True,
        verbose: bool = False,
        overwrite: bool = False,
    ) -> bool:
        success, item = self.get_item(
            datacube_id=datacube_id,
            log=verbose,
        )
        if not success:
            return success

        for asset_name in tqdm(list_of_assets):
            if asset_name not in item.assets:
                logger.error(
                    "{} not found in item.assets: {}".format(
                        asset_name,
                        ", ".join(list(item.assets.keys())),
                    )
                )
                success = False
                continue

            logger.info(f"ingesting {asset_name} ...")

            asset_relative_href = item.assets[asset_name].href
            logger.info(f"asset_relative_href= {asset_relative_href}")

            if verbose:
                for link in item.links:
                    logger.info("{}: {}".format(link.rel, link.href))

            root_href_candidates = [
                link.href for link in item.links if link.rel == "self"
            ]
            if not root_href_candidates:
                logger.error(
                    'cannot find "self" in {}.'.format(
                        ", ".join([link.rel for link in item.links])
                    )
                )
                success = False
                continue

            root_href = "/".join(root_href_candidates[0].split("/")[:-1])
            logger.info("root href: {}".format(root_href))

            asset_href = f"{root_href}/{asset_relative_href}"
            logger.info(f"asset_href: {asset_href}")

            suffixed_filename = self.get_filename(
                item=item,
                filename=asset_relative_href,
            )
            full_filename = str(
                pathlib.Path(
                    objects.path_of(
                        filename=suffixed_filename,
                        object_name=datacube_id,
                        create=True,
                    )
                ).resolve()
            )
            logger.info(f"filename: {full_filename}")

            if not file.download(
                asset_href,
                filename=full_filename,
                overwrite=overwrite,
            ):
                success = False
                continue

            if any(
                suffixed_filename.endswith(suffix) for suffix in raster_suffix
            ) and not log_geoimage(
                filename=suffixed_filename,
                object_name=datacube_id,
            ):
                success = False
                continue

        if not metadata.post_to_object(
            datacube_id,
            "item_info",
            item.to_dict(),
        ):
            success = False

        return success

    def query(
        self,
        collection_id: str,
        start_date: datetime.datetime,
        end_date: datetime.datetime,
        lat: float = -1,
        lon: float = -1,
        radius: float = env.MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS,
        count: int = -1,
        log: bool = False,
        verbose: bool = False,
    ) -> List[Any]:
        list_of_items = []

        collection = self.get_collection(
            collection_id=collection_id,
            log=verbose,
        )
        if collection is None:
            return list_of_items

        for child in tqdm(collection.get_children()):
            for item in child.get_items():
                item_date = datetime.datetime.strptime(
                    item.properties["datetime"],
                    (
                        "%Y-%m-%dT%H:%M:%SZ"
                        if "T" in item.properties["datetime"]
                        else "%Y-%m-%d %H:%M:%SZ"
                    ),
                )
                if start_date > item_date or item_date > end_date:
                    continue

                if lat != -1 and lon != -1:
                    polygon = Polygon(item.geometry["coordinates"][0])
                    point = Point(lon, lat)
                    if not point.within(polygon.buffer(radius)):
                        continue

                logger.info(f"{item.id} - {item_date}")
                list_of_items += [item]

                if count != -1 and len(list_of_items) >= count:
                    break

            if count != -1 and len(list_of_items) >= count:
                break

        if self.verbose or log:
            logger.info(f"{len(list_of_items):,} item(s).")

        return list_of_items



import os

if not QGIS_is_live:
    from logger import Q_log, Q_log_error


def Q_path_exists(path: str) -> bool:
    return os.path.exists(path) and os.path.isdir(path)


def Q_open_path(
    path: str,
    dryrun: bool = False,
) -> bool:
    if not Q_path_exists(path):
        Q_log_error("path not found.")
        return False

    Q_log(path)

    if not dryrun:
        os.system(f"open {path}")

    return True



import time
from typing import Union, List
import os
import random

if not QGIS_is_live:
    from .file_save import Q_save_text
    from .logger import Q_log

blue_geo_QGIS_path_server = os.path.join(
    os.getenv("HOME", ""),
    "Downloads/QGIS/server",
)

os.makedirs(blue_geo_QGIS_path_server, exist_ok=True)


def Q_seed(
    command: Union[str, List[str]],
    dryrun: bool = False,
    log: bool = True,
):
    if isinstance(command, list):
        command = " ".join(command)

    command_name = "{}-{:05d}".format(
        time.strftime("QGIS-command-%Y-%m-%d-%H-%M-%S", time.localtime(time.time())),
        random.randrange(100000),
    )

    if not dryrun:
        Q_save_text(
            filename=os.path.join(
                blue_geo_QGIS_path_server,
                f"{command_name}.command",
            ),
            text=[command],
            log=log,
        )

    if log:
        Q_log(f"{command} -> {command_name}", icon="🌱")
